// openpipe : You can play music by pipe or piano. In addition, you can talk to AI tutor to change the type of musical instrument or play the demo music.

#include <Wire.h>
#include "Adafruit_MPR121.h"
#include <SoftwareSerial.h>

#ifndef _BV
#define _BV(bit) (1 << (bit)) 
#endif

// You can have up to 4 on one i2c bus but one is enough for testing!
Adafruit_MPR121 cap = Adafruit_MPR121();

// Keeps track of the last pins touched
// so we know when buttons are 'released'
uint16_t lasttouched = 0;
uint16_t currtouched = 0;

// instrument shield setting
SoftwareSerial mySerial(2, 3); // RX, TX

// byte note = 60; //The MIDI note value to be played (old)
byte resetMIDI = 4; //Tied to VS1053 Reset line
byte ledPin = 13; //MIDI traffic inidicator
int  instrument = 66;

char receivedChar;
boolean newData = false;
int flag = 0; // set it that it play only once.
int tempo = 330;
byte note[] ={60,62,64,65,67,69,71,72}; // 設定MIDI 唱音數值（note value） - Do re Me Fa So La Ta Do //
byte inst[]={1,61,41,19,74,23,11,14};// 設定 8 種樂器號碼（music instrument code number）
int instMax=8; // 設定最多樂器數目
int dur=220;
int  ii=0;



void setup() {
  // Serial.begin(9600); // MPR121 original speed
  Serial.begin(57600);
  while (!Serial) { // needed to keep leonardo/micro from starting too fast!
    delay(10);
  }
  
  Serial.println("Adafruit MPR121 Capacitive Touch sensor test"); 
  
  // Default address is 0x5A, if tied to 3.3V its 0x5B
  // If tied to SDA its 0x5C and if SCL then 0x5D
  if (!cap.begin(0x5A)) {
    Serial.println("MPR121 not found, check wiring?");
    while (1);
  }
  Serial.println("MPR121 found!");

    //Setup soft serial for MIDI control
  mySerial.begin(31250);

 //Reset the VS1053
  pinMode(resetMIDI, OUTPUT);
  digitalWrite(resetMIDI, LOW);
  delay(100);
  digitalWrite(resetMIDI, HIGH);
  delay(100);
  talkMIDI(0xB0, 0x07, 120); //0xB0 is channel message, set channel volume to near max (127)
  
}

void loop() {
  // Get the currently touched pads  感應手指的觸覺
  currtouched = cap.touched();
 
  talkMIDI(0xC0, instrument, 0); //Set instrument number. 0xC0 is a 1 data byte command

  recvInfo();
  int voice_cmd = (receivedChar - '0');
  if (voice_cmd == 1)  { 
        //  AI();
        noteOn(0, note[0], 60);
        delay(200);
        noteOff(0, note[0], 60);
        delay(3);
  } else if(voice_cmd ==2) { // Instrument change 2.  改變樂器
        ii++;
        if (ii>instMax-1) {
        ii=0;}
        // 變換下一種樂器指令！
        talkMIDI(0xC0, inst[ii], 0); //Set instrument number. 0xC0 is a 1 data byte command 
    // 示範一下
        noteOn(0, note[ii], 60);
        delay(200);
        noteOff(0, note[ii], 60);
        delay(200);
  }     
   else if (voice_cmd ==3) { // play music 3. 播放音樂
       demo();
       delay(100);}
  
  for (uint8_t i=0; i<12; i++) {
    // it if *is* touched and *wasnt* touched before, alert!
    if ((currtouched & _BV(i)) && !(lasttouched & _BV(i)) ) {
      Serial.print(i); Serial.println(" touched");
      //Note on channel 1 (0x90), some note value (note), middle velocity (0x45):
      noteOn(0, note[i], 60);
      delay(50);
    }
    // if it *was* touched and now *isnt*, alert!
    if (!(currtouched & _BV(i)) && (lasttouched & _BV(i)) ) {
      Serial.print(i); Serial.println(" released");
      //Turn off the note with a given off/release velocity
      noteOff(0, note[i], 60);
      delay(50);
    }
  }

  // reset our state
  lasttouched = currtouched;

  // comment out this line for detailed data from the sensor!
  return;
  
  // debugging info, what
  Serial.print("\t\t\t\t\t\t\t\t\t\t\t\t\t 0x"); Serial.println(cap.touched(), HEX);
  Serial.print("Filt: ");
  for (uint8_t i=0; i<12; i++) {
    Serial.print(cap.filteredData(i)); Serial.print("\t");
  }
  Serial.println();
  Serial.print("Base: ");
  for (uint8_t i=0; i<12; i++) {
    Serial.print(cap.baselineData(i)); Serial.print("\t");
  }
  Serial.println();
  
  // put a delay so it isn't overwhelming
  delay(100);
}


//開始音樂  Send a MIDI note-on message.  Like pressing a piano key
//channel ranges from 0-15
void noteOn(byte channel, byte note1, byte attack_velocity) {
  talkMIDI( (0x90 | channel), note1, attack_velocity);
}

//關閉音樂  Send a MIDI note-off message.  Like releasing a piano key
void noteOff(byte channel, byte note1, byte release_velocity) {
  talkMIDI( (0x80 | channel), note1, release_velocity);
}


//Plays a MIDI note播放音符. Doesn't check to see that cmd is greater than 127, or that data values are less than 127
void talkMIDI(byte cmd, byte data1, byte data2) {
  digitalWrite(ledPin, HIGH);
  mySerial.write(cmd);
  mySerial.write(data1);

  //Some commands only have one data byte. All cmds less than 0xBn have 2 data bytes 
  //(sort of: http://253.ccarh.org/handout/midiprotocol/)
  if( (cmd & 0xF0) <= 0xB0)
    mySerial.write(data2);

  digitalWrite(ledPin, LOW);
}

// receive 接收 the AI tutor 控制訊號 control signal!
void recvInfo() {

  if (Serial.available() > 0) {

    receivedChar = Serial.read();
    newData = true;
    
  }
}

// Play the song demo...
void demo()  
{
  
int qq = 47; // twinkle twinkle little star total 48 (0-47)notes
char dnotes[] = "ccggaag ffeeddc ggffeed ggffeed ccggaag ffeeddc "; // a space represents a rest DEMO 歌的音符
// all 1 tempo of twinkle twinkle little star !!!!!
//
int rr = 0;
int dn = 0; // demo note number 
  

for (int rr = 0; rr < qq; rr++) {
if (dnotes[rr]=='c') { dn=0;}
if (dnotes[rr]=='d') { dn=1;}
if (dnotes[rr]=='e') { dn=2;}
if (dnotes[rr]=='f') { dn=3;}
if (dnotes[rr]=='g') { dn=4;}
if (dnotes[rr]=='a') { dn=5;} 
if (dnotes[rr]=='b') { dn=6;}
if (dnotes[rr]=='C') { dn=7;}
      
    if (dnotes[rr] == ' ') {
     //LEDoff();
      delay(tempo); // rest
    } else {
      noteOn(0, note[dn], tempo);
      delay(200);
      noteOff(0, note[dn], tempo);
    }

  }   

}

